@@ -3,10 +3,9 @@
 // Distributed under the Boost Software License, Version 1.0. 
 // See http://www.boost.org/LICENSE_1_0.txt
 
-use std::sync::{Arc, Weak, Mutex};
+use std::sync::{Arc, Weak, atomic::{AtomicBool, AtomicUsize, Ordering}};
 
-use crate::Signal;
-use crate::signal_core::UntypedSignalCore;
+use crate::{Signal, ConnectHandle};
 use crate::combiner::Combiner;
 
 /// Represents a position to connect a slot to in a group of slots.
@@ -25,14 +24,14 @@ pub enum Group<G>
 where
     G: Ord + Send + Sync
 {
-    /// The unnamed "front" group. Slots in the "front" grouped are executed before
+    /// The unnamed "front" group. Slots in the "front" group are executed before
     /// slots from named groups and the unnamed "back" group.
     Front,
     /// A named group. Slots in named groups are executed after all slots in the
     /// unnamed "front" group. Individual named groups are executed according the ordering defined by the
     /// [Ord] trait.
     Named(G),
-    /// The unnamed "back" group. Slots in the "back" grouped are executed after
+    /// The unnamed "back" group. Slots in the "back" group are executed after
     /// slots from the unnamed "front" group and slots from named groups.
     Back
 }
@@ -124,10 +123,22 @@ macro_rules! impl_connect {
             where
                 F: Fn($($args,)*) -> R + Send + Sync + 'static
             {
-                let untyped_core: Arc<dyn UntypedSignalCore> = self.core.clone();
-                let make_conn = |id| Connection::new(Arc::downgrade(&untyped_core), id);
+                let weak_core = Arc::downgrade(&self.core);
 
-                let mut lock = self.core.lock().unwrap();
+                let cleanup = move || {
+                    if let Some(core) = weak_core.upgrade() {
+                        let mut lock = core.write().unwrap();
+                        let mut core_clone = (**lock).clone();
+                        core_clone.cleanup();
+                        *lock = Arc::new(core_clone);
+                    }
+                };
+
+                let make_conn = move |connected, blocker_count| {
+                    Connection::new(connected, blocker_count, Arc::new(cleanup))
+                };
+
+                let mut lock = self.core.write().unwrap();
                 let mut core_clone = (**lock).clone();
 
                 let wrapped_f = move |($($params,)*)| f($($params,)*);
@@ -141,10 +152,22 @@ macro_rules! impl_connect {
             where
                 F: Fn(Connection, $($args,)*) -> R + Send + Sync + 'static
             {
-                let untyped_core: Arc<dyn UntypedSignalCore> = self.core.clone();
-                let make_conn = |id| Connection::new(Arc::downgrade(&untyped_core), id);
+                let weak_core = Arc::downgrade(&self.core);
 
-                let mut lock = self.core.lock().unwrap();
+                let cleanup = move || {
+                    if let Some(core) = weak_core.upgrade() {
+                        let mut lock = core.write().unwrap();
+                        let mut core_clone = (**lock).clone();
+                        core_clone.cleanup();
+                        *lock = Arc::new(core_clone);
+                    }
+                };
+
+                let make_conn = move |connected, blocker_count| {
+                    Connection::new(connected, blocker_count, Arc::new(cleanup))
+                };
+
+                let mut lock = self.core.write().unwrap();
                 let mut core_clone = (**lock).clone();
 
                 let wrapped_f = move |conn, ($($params,)*)| f(conn, $($params,)*);
@@ -154,6 +177,35 @@ macro_rules! impl_connect {
                 conn
             }
         }
+
+        // Implement Connect traits for COnnectHandle
+        impl<R, C, G, $($args,)*> $name<R, C, G, $($args,)*> for ConnectHandle<($($args,)*), R, C, G> 
+        where
+            ($($args,)*): Clone + 'static,
+            R: 'static,
+            C: Combiner<R> + 'static,
+            G: Ord + Send + Sync + 'static,
+        {
+            fn connect_group_position<F>(&self, f: F, group: Group<G>, pos: Position) -> Connection
+            where
+                F: Fn($($args,)*) -> R + Send + Sync + 'static
+            {
+                self.weak_sig
+                    .upgrade()
+                    .map(|sig| sig.connect_group_position(f, group, pos))
+                    .unwrap_or(Connection::empty())
+            }
+
+            fn connect_group_position_extended<F>(&self, f: F, group: Group<G>, pos: Position) -> Connection
+            where
+                F: Fn(Connection, $($args,)*) -> R + Send + Sync + 'static
+            {
+                self.weak_sig
+                    .upgrade()
+                    .map(|sig| sig.connect_group_position_extended(f, group, pos))
+                    .unwrap_or(Connection::empty())
+            }
+        }
     };
 }
 
@@ -176,64 +228,69 @@ impl_connect!(Connect12; T0 T1 T2 T3 T4 T5 T6 T7 T8 T9 T10 T11; a b c d e f g h
 #[derive(Clone)]
 pub struct ConnectionImpl<const SCOPED: bool>
 {
-    weak_core: Weak<dyn UntypedSignalCore>,
-    slot_id: usize
+    weak_connected: Weak<AtomicBool>,
+    weak_blocker_count: Weak<AtomicUsize>,
+    cleanup: Arc<dyn Fn() -> () + Send + Sync>
 }
 
 impl<const SCOPED: bool> ConnectionImpl<SCOPED> {
-    fn new(weak_core: Weak<dyn UntypedSignalCore>, slot_id: usize) -> Self {
+    fn new(weak_connected: Weak<AtomicBool>, weak_blocker_count: Weak<AtomicUsize>, cleanup: Arc<dyn Fn() -> () + Send + Sync>) -> Self {
         Self {
-            weak_core,
-            slot_id
+            weak_connected,
+            weak_blocker_count,
+            cleanup
+        }
+    }
+
+    fn empty() -> Self {
+        Self {
+            weak_connected: Weak::new(),
+            weak_blocker_count: Weak::new(),
+            cleanup: Arc::new(|| ())
         }
     }
 
     /// Returns true if the underlying slot is still connected, false otherwise. Will return false 
     /// if the underlying signal no longer exists.
     pub fn connected(&self) -> bool {
-        match self.weak_core.upgrade() {
-            Some(core) => {
-                core.connected(self.slot_id)
-            }
-            None => false
-        }
+        self.weak_connected
+            .upgrade()
+            .map(|connected| connected.load(Ordering::SeqCst))
+            .unwrap_or(false)
     }
 
     /// Disconnects the underlying slot. Further, repeated calls to `disconnect` will do nothing.
     /// When a connection is disconnected its underlying slot is permanently removed from the the signal's slot list.
     /// Once disconnected, there is no way to re-connect a slot.
     pub fn disconnect(&self) {
-        if let Some(core) = self.weak_core.upgrade() {
-            core.disconnect(self.slot_id);
+        if let Some(connected) = self.weak_connected.upgrade() {
+            connected.store(false, Ordering::SeqCst);
+            (self.cleanup)();
         }
     }
 
     /// Returns true if the underlying slot is blocked, false otherwise. Will return true if either the
     /// underyling slot or underlying signal no longer exists.
     pub fn blocked(&self) -> bool {        
-        match self.weak_core.upgrade() {
-            Some(core) => {
-                core.blocked(self.slot_id)
-            }
-            None => true
-        }
+        self.weak_blocker_count
+            .upgrade()
+            .map(|blocker_count| blocker_count.load(Ordering::SeqCst) != 0usize)
+            .unwrap_or(true)
     }
 
     /// Returns the number of [SharedConnectionBlocks](SharedConnectionBlock) currently blocking the slot. 
     /// Will return `usize::Max` if either the underyling slot or underlying signal no longer exists.
     pub fn blocker_count(&self) -> usize {
-        match self.weak_core.upgrade() {
-            Some(core) => {
-                core.blocker_count(self.slot_id)
-            }
-            None => usize::MAX
-        }
+        self.weak_blocker_count
+            .upgrade()
+            .map(|blocker_count| blocker_count.load(Ordering::SeqCst))
+            .unwrap_or(usize::MAX)
     }
 
     #[must_use="shared connection blocks are automatically unblocked when dropped"]
     /// Gets a [SharedConnectionBlock] that can be used to temporarily block the underlying slot.
     pub fn shared_block(&self, initially_blocking: bool) -> SharedConnectionBlock {
-        SharedConnectionBlock::new(self.weak_core.clone(), self.slot_id, initially_blocking)
+        SharedConnectionBlock::new(self.weak_blocker_count.clone(), initially_blocking)
     }
 }
 
@@ -250,7 +307,7 @@ impl ConnectionImpl<false> {
     /// Consumes the connection and returns a [ScopedConnection].
     #[must_use="ScopedConnection automatically disconnects when dropped"]
     pub fn scoped(self) -> ScopedConnection {
-        ScopedConnection::new(self.weak_core.clone(), self.slot_id)
+        ScopedConnection::new(self.weak_connected.clone(), self.weak_blocker_count.clone(), self.cleanup.clone())
     }
 }
 
@@ -333,17 +390,15 @@ pub type ScopedConnection = ConnectionImpl<true>;
 /// assert_eq!(sig.emit(), Some(4)); // blocker was dropped
 /// ```
 pub struct SharedConnectionBlock {
-    weak_core: Weak<dyn UntypedSignalCore>,
-    slot_id: usize,
-    blocking: Mutex<bool>
+    weak_blocker_count: Weak<AtomicUsize>,
+    blocking: AtomicBool
 }
 
 impl SharedConnectionBlock {
-    fn new(weak_core: Weak<dyn UntypedSignalCore>, slot_id: usize, initially_blocking: bool) -> Self {
+    fn new(weak_blocker_count: Weak<AtomicUsize>, initially_blocking: bool) -> Self {
         let shared_block = Self {
-            weak_core,
-            slot_id,
-            blocking: Mutex::new(false)
+            weak_blocker_count,
+            blocking: AtomicBool::new(false)
         };
 
         if initially_blocking {
@@ -373,23 +428,26 @@ impl SharedConnectionBlock {
     /// slot will be executed when the signal is emitted because there could be other existing blockers for
     /// the slot.
     pub fn blocking(&self) -> bool {
-        *self.blocking.lock().unwrap()
+        self.blocking.load(Ordering::SeqCst)
     }
 
     fn block_impl(&self, block: bool) {
-        if let Some(core) = self.weak_core.upgrade() {
-            core.block(self.slot_id, block);
+        if let Some(blocker_count) = self.weak_blocker_count.upgrade() {
+            if block {
+                blocker_count.fetch_add(1, Ordering::SeqCst);
+            } else {
+                blocker_count.fetch_sub(1, Ordering::SeqCst);
+            }
         }
 
-        let mut lock = self.blocking.lock().unwrap();
-        *lock = block;
+        self.blocking.store(block, Ordering::SeqCst);
     }
 }
 
 impl Clone for SharedConnectionBlock {
     /// Creates a copy of the given `SharedConnectionBlock` with the same blocking state.
     fn clone(&self) -> Self {
-        SharedConnectionBlock::new(self.weak_core.clone(), self.slot_id, self.blocking())
+        SharedConnectionBlock::new(self.weak_blocker_count.clone(), self.blocking())
     }
 }
 
