@@ -16,10 +16,11 @@
 //!
 //! ### Links
 //! * [Github](https://github.com/christiandaley/signals2/)
+//! * [Changelog](https://github.com/christiandaley/signals2/blob/master/CHANGES.md)
 
 #![deny(missing_docs)]
 
-use std::sync::{Arc, Weak, Mutex};
+use std::sync::{Arc, Weak, RwLock};
 
 mod signal_core;
 use signal_core::{SignalCore};
@@ -90,7 +91,7 @@ where
     C: Combiner<R> + 'static,
     G: Ord + Send + Sync + 'static
 {
-    core: Arc<Mutex<Arc<SignalCore<Args, R, C, G>>>>
+    core: Arc<RwLock<Arc<SignalCore<Args, R, C, G>>>>
 }
 
 impl<Args, R, C, G> Clone for Signal<Args, R, C, G>
@@ -131,7 +132,7 @@ where
     pub fn new_with_combiner(combiner: C) -> Self {
         let core: SignalCore<Args, R, C, G> = SignalCore::new(combiner);
         Signal {
-            core: Arc::new(Mutex::new(Arc::new(core)))
+            core: Arc::new(RwLock::new(Arc::new(core)))
         }
     }
 
@@ -142,9 +143,23 @@ where
         }
     }
 
+    /// Creates a [ConnectHandle] that can be used to connect new slots to the signal.
+    pub fn get_connect_handle(&self) -> ConnectHandle<Args, R, C, G> {
+        ConnectHandle {
+            weak_sig: self.weak()
+        }
+    }
+
+    /// Creates an [EmitHandle] that can be used to emit the signal.
+    pub fn get_emit_handle(&self) -> EmitHandle<Args, R, C, G> {
+        EmitHandle {
+            weak_sig: self.weak()
+        }
+    }
+
     /// Sets a new [Combiner] for the signal.
     pub fn set_combiner(&self, combiner: C) {
-        let mut lock = self.core.lock().unwrap();
+        let mut lock = self.core.write().unwrap();
         let mut new_core = (**lock).clone();
         new_core.set_combiner(combiner);
         *lock = Arc::new(new_core);
@@ -153,8 +168,8 @@ where
     /// Disconnects all slots from the signal. Will cause any existing [Connections](Connection) to enter a
     /// "disconnected" state.
     pub fn clear(&self) {
-        self.core.lock().unwrap().disconnect_all();
-        let mut lock = self.core.lock().unwrap();
+        self.core.read().unwrap().disconnect_all();
+        let mut lock = self.core.write().unwrap();
         let mut new_core = (**lock).clone();
         new_core.clear();
         *lock = Arc::new(new_core);
@@ -162,8 +177,7 @@ where
 
     /// Returns the number of connected slots for the signal.
     pub fn count(&self) -> usize {
-        let lock = self.core.lock().unwrap();
-        lock.count()
+        self.core.read().unwrap().count()
     }
 }
 
@@ -208,7 +222,7 @@ where
     C: Combiner<R> + 'static,
     G: Ord + Send + Sync + 'static
 {
-    weak_core: Weak<Mutex<Arc<SignalCore<Args, R, C, G>>>>
+    weak_core: Weak<RwLock<Arc<SignalCore<Args, R, C, G>>>>
 }
 
 impl<Args, R, C, G> Clone for WeakSignal<Args, R, C, G>
@@ -239,3 +253,60 @@ where
         self.weak_core.upgrade().map(|core| Signal {core})
     }
 }
+
+/// A handle to a signal that allows new slots to be connected to the underlying signal.
+/// Useful in cases where it is undesireable to allow unresitriced access to a signal while
+/// still allowing new slots to be connected. Internally, a `ConnectHandle` uses a [WeakSignal].
+/// If the underlying signal no longer exists, `connect` will return a connection that is in a 
+/// disconnected state.
+/// # Example
+/// ```
+/// use signals2::*;
+/// 
+/// let sig: Signal<(), i32> = Signal::new();
+/// let connect_handle = sig.get_connect_handle();
+/// let conn = connect_handle.connect(|| 1);
+/// assert!(conn.connected());
+/// assert_eq!(sig.emit(), Some(1));
+///
+/// std::mem::drop(sig);
+/// let conn = connect_handle.connect(|| 2);
+/// assert!(!conn.connected());
+/// ```
+pub struct ConnectHandle<Args, R = (), C = DefaultCombiner, G = i32>
+where 
+    Args: Clone + 'static,
+    R: 'static,
+    C: Combiner<R> + 'static,
+    G: Ord + Send + Sync + 'static
+{
+    weak_sig: WeakSignal<Args, R, C, G>
+}
+
+
+/// A handle to a signal that allows the signal to be emitted. Useful in cases where it is 
+/// undesireable to allow unresitriced access to a signal while still allowing the signal to be 
+/// emitted. Internally, an `EmitHandle` uses a [WeakSignal]. The result of calling `emit` on an
+/// `EmitHandle` is an `Option<C::Output>` where `C` is the combiner type of the signal. If the
+/// underlying signal no longer exists, `None` is returned.
+/// # Example
+/// ```
+/// use signals2::*;
+/// 
+/// let sig: Signal<(), i32> = Signal::new();
+/// let emit_handle = sig.get_emit_handle();
+/// sig.connect(|| 1);
+/// assert_eq!(emit_handle.emit(), Some(Some(1)));
+///
+/// std::mem::drop(sig);
+/// assert_eq!(emit_handle.emit(), None);
+/// ```
+pub struct EmitHandle<Args, R = (), C = DefaultCombiner, G = i32>
+where 
+    Args: Clone + 'static,
+    R: 'static,
+    C: Combiner<R> + 'static,
+    G: Ord + Send + Sync + 'static
+{
+    weak_sig: WeakSignal<Args, R, C, G>
+}
\ No newline at end of file
